@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2021 NVIDIA Corporation
+    Copyright (c) 2016-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,6 +21,7 @@
 
 *******************************************************************************/
 #include "uvm_hal.h"
+#include "uvm_global.h"
 #include "uvm_gpu.h"
 #include "uvm_kvmalloc.h"
 #include "uvm_mem.h"
@@ -44,10 +45,10 @@ static NvU32 first_page_size(NvU32 page_sizes)
 
 static inline NV_STATUS __alloc_map_sysmem(NvU64 size, uvm_gpu_t *gpu, uvm_mem_t **sys_mem)
 {
-    if (g_uvm_global.sev_enabled)
+    if (g_uvm_global.conf_computing_enabled)
         return uvm_mem_alloc_sysmem_dma_and_map_cpu_kernel(size, gpu, current->mm, sys_mem);
-    else
-        return uvm_mem_alloc_sysmem_and_map_cpu_kernel(size, current->mm, sys_mem);
+
+    return uvm_mem_alloc_sysmem_and_map_cpu_kernel(size, current->mm, sys_mem);
 }
 
 static NV_STATUS check_accessible_from_gpu(uvm_gpu_t *gpu, uvm_mem_t *mem)
@@ -335,9 +336,6 @@ error:
 
 static bool should_test_page_size(size_t alloc_size, NvU32 page_size)
 {
-    if (g_uvm_global.sev_enabled)
-        return false;
-
     if (g_uvm_global.num_simulated_devices == 0)
         return true;
 
@@ -371,7 +369,7 @@ static NV_STATUS test_all(uvm_va_space_t *va_space)
 
     // TODO: Bug 3839176: the test is waived on Confidential Computing because
     // it assumes that GPU can access system memory without using encryption.
-    if (uvm_conf_computing_mode_enabled(uvm_va_space_find_first_gpu(va_space)))
+    if (g_uvm_global.conf_computing_enabled)
         return NV_OK;
 
     gpu_count = uvm_processor_mask_get_gpu_count(&va_space->registered_gpus);
@@ -522,7 +520,7 @@ static NV_STATUS test_basic_vidmem_unprotected(uvm_gpu_t *gpu)
     // If CC is disabled, the behavior should be identical to that of a
     // protected allocation.
     params.is_unprotected = true;
-    if (uvm_conf_computing_mode_enabled(gpu))
+    if (g_uvm_global.conf_computing_enabled)
         TEST_CHECK_RET(uvm_mem_alloc(&params, &mem) == NV_ERR_NO_MEMORY);
     else
         TEST_NV_CHECK_RET(uvm_mem_alloc(&params, &mem));
@@ -578,7 +576,7 @@ static NV_STATUS test_basic_dma_pool(uvm_gpu_t *gpu)
 
     // If the Confidential Computing feature is disabled, the DMA buffers
     // pool is not initialized.
-    if (!uvm_conf_computing_mode_enabled(gpu))
+    if (!g_uvm_global.conf_computing_enabled)
         return NV_OK;
 
     // We're going to reclaim one more chunks that the pool have. Triggerring
