@@ -15,7 +15,7 @@
     IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
     FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
     THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-    LIABILITY, WHETHER IN AN hint OF CONTRACT, TORT OR OTHERWISE, ARISING
+    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
     FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     DEALINGS IN THE SOFTWARE.
 
@@ -25,6 +25,7 @@
 #include "uvm_perf_events.h"
 #include "uvm_perf_module.h"
 #include "uvm_perf_prefetch.h"
+#include "uvm_perf_utils.h"
 #include "uvm_kvmalloc.h"
 #include "uvm_va_block.h"
 #include "uvm_va_range.h"
