@@ -40,10 +40,9 @@
 
 static NvU32 get_push_begin_size(uvm_channel_t *channel)
 {
-    if (uvm_channel_is_sec2(channel)) {
-        // SEC2 channels allocate CSL signature buffer at the beginning.
+    // SEC2 channels allocate CSL signature buffer at the beginning.
+    if (uvm_channel_is_sec2(channel))
         return UVM_CONF_COMPUTING_SIGN_BUF_MAX_SIZE + UVM_METHOD_SIZE;
-    }
 
     return 0;
 }
@@ -51,10 +50,12 @@ static NvU32 get_push_begin_size(uvm_channel_t *channel)
 // This is the storage required by a semaphore release.
 static NvU32 get_push_end_min_size(uvm_channel_t *channel)
 {
-    if (uvm_channel_is_ce(channel)) {
-        if (uvm_channel_is_wlc(channel)) {
-            // Space (in bytes) used by uvm_push_end() on a Secure CE channel.
-            // Note that Secure CE semaphore release pushes two memset and one
+    if (g_uvm_global.conf_computing_enabled) {
+        if (uvm_channel_is_ce(channel)) {
+            // Space (in bytes) used by uvm_push_end() on a CE channel when
+            // the Confidential Computing feature is enabled.
+            //
+            // Note that CE semaphore release pushes two memset and one
             // encryption method on top of the regular release.
             // Memset size
             // -------------
@@ -75,43 +76,44 @@ static NvU32 get_push_end_min_size(uvm_channel_t *channel)
             //
             // TOTAL                            : 144 Bytes
 
-            // Same as CE + LCIC GPPut update + LCIC doorbell
-            return 24 + 144 + 24 + 24;
-        }
-        else if (uvm_channel_is_secure_ce(channel)) {
+            if (uvm_channel_is_wlc(channel)) {
+                // Same as CE + LCIC GPPut update + LCIC doorbell
+                return 24 + 144 + 24 + 24;
+            }
+
             return 24 + 144;
         }
-        // Space (in bytes) used by uvm_push_end() on a CE channel.
-        return 24;
-    }
-    else if (uvm_channel_is_sec2(channel)) {
+
+        UVM_ASSERT(uvm_channel_is_sec2(channel));
+
         // A perfectly aligned inline buffer in SEC2 semaphore release.
         // We add UVM_METHOD_SIZE because of the NOP method to reserve
         // UVM_CSL_SIGN_AUTH_TAG_SIZE_BYTES (the inline buffer.)
         return 48 + UVM_CSL_SIGN_AUTH_TAG_SIZE_BYTES + UVM_METHOD_SIZE;
     }
 
-    return 0;
+    UVM_ASSERT(uvm_channel_is_ce(channel));
+
+    // Space (in bytes) used by uvm_push_end() on a CE channel.
+    return 24;
 }
 
 static NvU32 get_push_end_max_size(uvm_channel_t *channel)
 {
-    if (uvm_channel_is_ce(channel)) {
-        if (uvm_channel_is_wlc(channel)) {
-            // WLC pushes are always padded to UVM_MAX_WLC_PUSH_SIZE
-            return UVM_MAX_WLC_PUSH_SIZE;
-        }
-        // Space (in bytes) used by uvm_push_end() on a CE channel.
-        return get_push_end_min_size(channel);
-    }
-    else if (uvm_channel_is_sec2(channel)) {
-        // Space (in bytes) used by uvm_push_end() on a SEC2 channel.
-        // Note that SEC2 semaphore release uses an inline buffer with alignment
-        // requirements. This is the "worst" case semaphore_release storage.
-        return 48 + UVM_CSL_SIGN_AUTH_TAG_SIZE_BYTES + UVM_CONF_COMPUTING_AUTH_TAG_ALIGNMENT;
-    }
+    // WLC pushes are always padded to UVM_MAX_WLC_PUSH_SIZE
+    if (uvm_channel_is_wlc(channel))
+        return UVM_MAX_WLC_PUSH_SIZE;
 
-    return 0;
+    // Space (in bytes) used by uvm_push_end() on a SEC2 channel.
+    // Note that SEC2 semaphore release uses an inline buffer with alignment
+    // requirements. This is the "worst" case semaphore_release storage.
+    if (uvm_channel_is_sec2(channel))
+        return 48 + UVM_CSL_SIGN_AUTH_TAG_SIZE_BYTES + UVM_CONF_COMPUTING_AUTH_TAG_ALIGNMENT;
+
+    UVM_ASSERT(uvm_channel_is_ce(channel));
+
+    // Space (in bytes) used by uvm_push_end() on a CE channel.
+    return get_push_end_min_size(channel);
 }
 
 static NV_STATUS test_push_end_size(uvm_va_space_t *va_space)
@@ -128,11 +130,11 @@ static NV_STATUS test_push_end_size(uvm_va_space_t *va_space)
             NvU32 push_end_size_expected[2];
 
             // SEC2 is only available when Confidential Computing is enabled
-            if ((type == UVM_CHANNEL_TYPE_SEC2) && !uvm_conf_computing_mode_enabled(gpu))
+            if ((type == UVM_CHANNEL_TYPE_SEC2) && !g_uvm_global.conf_computing_enabled)
                 continue;
 
             // WLC is only available when Confidential Computing is enabled
-            if ((type == UVM_CHANNEL_TYPE_WLC) && !uvm_conf_computing_mode_enabled(gpu))
+            if ((type == UVM_CHANNEL_TYPE_WLC) && !g_uvm_global.conf_computing_enabled)
                 continue;
 
             // LCIC doesn't accept pushes
@@ -191,7 +193,7 @@ static NV_STATUS test_push_inline_data_gpu(uvm_gpu_t *gpu)
 
     // TODO: Bug 3839176: test is waived on Confidential Computing because
     // it assumes that GPU can access system memory without using encryption.
-    if (uvm_conf_computing_mode_enabled(gpu))
+    if (g_uvm_global.conf_computing_enabled)
         return NV_OK;
 
     status = uvm_mem_alloc_sysmem_and_map_cpu_kernel(UVM_PUSH_INLINE_DATA_MAX_SIZE, current->mm, &mem);
@@ -294,10 +296,17 @@ static NV_STATUS test_concurrent_pushes(uvm_va_space_t *va_space)
 {
     NV_STATUS status = NV_OK;
     uvm_gpu_t *gpu;
-    NvU32 i;
     uvm_push_t *pushes;
-    uvm_tracker_t tracker = UVM_TRACKER_INIT();
-    uvm_channel_type_t channel_type = UVM_CHANNEL_TYPE_GPU_INTERNAL;
+    uvm_tracker_t tracker;
+
+    // When the Confidential Computing feature is enabled, a channel reserved at
+    // the start of a push cannot be reserved again until that push ends. The
+    // test is waived, because the number of pushes it starts per pool exceeds
+    // the number of channels in the pool, so it would block indefinitely.
+    if (g_uvm_global.conf_computing_enabled)
+        return NV_OK;
+
+    uvm_tracker_init(&tracker);
 
     // As noted above, this test does unsafe things that would be detected by
     // lock tracking, opt-out.
@@ -310,16 +319,11 @@ static NV_STATUS test_concurrent_pushes(uvm_va_space_t *va_space)
     }
 
     for_each_va_space_gpu(gpu, va_space) {
+        NvU32 i;
 
-        // A secure channels reserved at the start of a push cannot be reserved
-        // again until that push ends. The test would block indefinitely
-        // if secure pools are not skipped, because the number of pushes started
-        // per pool exceeds the number of channels in the pool.
-        if (uvm_channel_type_requires_secure_pool(gpu, channel_type))
-            goto done;
         for (i = 0; i < UVM_PUSH_MAX_CONCURRENT_PUSHES; ++i) {
             uvm_push_t *push = &pushes[i];
-            status = uvm_push_begin(gpu->channel_manager, channel_type, push, "concurrent push %u", i);
+            status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_GPU_INTERNAL, push, "concurrent push %u", i);
             TEST_CHECK_GOTO(status == NV_OK, done);
         }
         for (i = 0; i < UVM_PUSH_MAX_CONCURRENT_PUSHES; ++i) {
@@ -376,7 +380,7 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
 
     // TODO: Bug 3839176: test is waived on Confidential Computing because
     // it assumes that GPU can access system memory without using encryption.
-    if (uvm_conf_computing_mode_enabled(gpu))
+    if (g_uvm_global.conf_computing_enabled)
         return NV_OK;
 
     // This test issues virtual memcopies/memsets, which in SR-IOV heavy cannot
@@ -594,13 +598,12 @@ static NV_STATUS test_max_pushes_on_gpu(uvm_gpu_t *gpu)
     TEST_CHECK_GOTO(status == NV_OK, done);
 
     uvm_gpu_semaphore_set_payload(&sema, 0);
-    if (uvm_conf_computing_mode_enabled(gpu)) {
-        // Use SEC2 channel when Confidential Compute is enabled
-        // since all other channel types need extra space for
-        // work launch, and the channel type really doesn't
-        // matter for this test.
+
+    // Use SEC2 channel when Confidential Compute is enabled since all other
+    // channel types need extra space for work launch, and the channel type
+    // really doesn't matter for this test.
+    if (g_uvm_global.conf_computing_enabled)
         channel_type = UVM_CHANNEL_TYPE_SEC2;
-    }
 
     // Need to wait for all channels to completely idle so that the pushbuffer
     // is in completely idle state when we begin.
@@ -673,13 +676,11 @@ static NV_STATUS test_idle_chunks_on_gpu(uvm_gpu_t *gpu)
     NvU32 i;
     uvm_channel_type_t channel_type = UVM_CHANNEL_TYPE_GPU_INTERNAL;
 
-    if (uvm_conf_computing_mode_enabled(gpu)) {
-        // Use SEC2 channel when Confidential Compute is enabled
-        // since all other channel types need extra space for
-        // work launch, and the channel type really doesn't
-        // matter for this test.
+    // Use SEC2 channel when Confidential Compute is enabled since all other
+    // channel types need extra space for work launch, and the channel type
+    // really doesn't matter for this test.
+    if (g_uvm_global.conf_computing_enabled)
         channel_type = UVM_CHANNEL_TYPE_SEC2;
-    }
 
     uvm_tracker_init(&tracker);
 
@@ -872,17 +873,17 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
     NvU32 i;
     NV_STATUS status;
     uvm_gpu_t *gpu, *gpu_a, *gpu_b;
-    uvm_mem_t *mem[UVM_ID_MAX_PROCESSORS] = {NULL};
+    uvm_mem_t **mem;
     NvU32 *host_ptr;
     const size_t size = 1024 * 1024;
     bool waive = true;
 
-    for_each_va_space_gpu(gpu_a, va_space) {
+    // TODO: Bug 3839176: the test is waived on Confidential Computing because
+    // it assumes that GPU can access system memory without using encryption.
+    if (g_uvm_global.conf_computing_enabled)
+        return NV_OK;
 
-        // TODO: Bug 3839176: the test is waived on Confidential Computing because
-        // it assumes that GPU can access system memory without using encryption.
-        if (uvm_conf_computing_mode_enabled(gpu_a))
-            return NV_OK;
+    for_each_va_space_gpu(gpu_a, va_space) {
         for_each_va_space_gpu(gpu_b, va_space) {
             if (can_do_peer_copies(va_space, gpu_a, gpu_b)) {
                 waive = false;
@@ -894,6 +895,10 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
     if (waive)
         return NV_OK;
 
+    mem = uvm_kvmalloc_zero(sizeof(*mem) * UVM_ID_MAX_PROCESSORS);
+    if (!mem)
+        return NV_ERR_NO_MEMORY;
+
     // Alloc and initialize host buffer
     status = uvm_mem_alloc_sysmem_and_map_cpu_kernel(size, current->mm, &mem[UVM_ID_CPU_VALUE]);
     TEST_CHECK_GOTO(status == NV_OK, done);
@@ -961,6 +966,7 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
         uvm_mem_free(mem[uvm_id_value(gpu->id)]);
 
     uvm_mem_free(mem[UVM_ID_CPU_VALUE]);
+    uvm_kvfree(mem);
 
     return status;
 }
