@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -559,6 +559,76 @@ nvswitch_soe_disable_nport_fatal_interrupts_ls10
     }
 }
 
+/*
+ * @Brief : Perform register writes in SOE during TNVL
+ *
+ * @param[in] device
+ * @param[in] offset
+ * @param[in] data
+ */
+NvlStatus
+nvswitch_soe_reg_wr_32_ls10
+(
+    nvswitch_device *device,
+    NvU32           offset,
+    NvU32           data
+)
+{
+    FLCN *pFlcn;
+    NvU32               cmdSeqDesc = 0;
+    NV_STATUS           status;
+    RM_FLCN_CMD_SOE     cmd;
+    NVSWITCH_TIMEOUT    timeout;
+    RM_SOE_TNVL_CMD_REGISTER_WRITE *pRegisterWrite;
+    NVSWITCH_GET_BIOS_INFO_PARAMS params = { 0 };
+
+    if (!nvswitch_is_soe_supported(device))
+    {
+        NVSWITCH_PRINT(device, INFO,
+            "%s: SOE is not supported\n",
+            __FUNCTION__);
+        return NVL_SUCCESS; // -NVL_ERR_NOT_SUPPORTED
+    }
+
+    status = device->hal.nvswitch_ctrl_get_bios_info(device, &params);
+    if ((status != NVL_SUCCESS) || ((params.version & SOE_VBIOS_VERSION_MASK) < 
+            SOE_VBIOS_REVLOCK_ISSUE_REGISTER_WRITE))
+    {
+        nvswitch_reg_write_32(device, offset, data);
+        return NVL_SUCCESS;
+    }
+
+    pFlcn = device->pSoe->pFlcn;
+
+    nvswitch_os_memset(&cmd, 0, sizeof(cmd));
+
+    cmd.hdr.unitId = RM_SOE_UNIT_TNVL;
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(TNVL, REGISTER_WRITE);
+ 
+    pRegisterWrite = &cmd.cmd.tnvl.registerWrite;
+    pRegisterWrite->cmdType = RM_SOE_TNVL_CMD_ISSUE_REGISTER_WRITE;
+    pRegisterWrite->offset = offset;
+    pRegisterWrite->data = data;
+
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
+    status = flcnQueueCmdPostBlocking(device, pFlcn,
+                                      (PRM_FLCN_CMD)&cmd,
+                                      NULL,                 // pMsg
+                                      NULL,                 // pPayload
+                                      SOE_RM_CMDQ_LOG_ID,
+                                      &cmdSeqDesc,
+                                      &timeout);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to send REGISTER_WRITE command to SOE, offset = 0x%x, data = 0x%x\n", 
+            __FUNCTION__, offset, data);
+        return -NVL_ERR_GENERIC;
+    }
+
+    return NVL_SUCCESS;
+}
+
 /*
  * @Brief : Init sequence for SOE FSP RISCV image
  *
@@ -609,14 +679,21 @@ nvswitch_init_soe_ls10
     }
 
     // Register SOE callbacks
-    status = nvswitch_soe_register_event_callbacks(device);
-    if (status != NVL_SUCCESS)
+    if (!nvswitch_is_tnvl_mode_enabled(device))
     {
-        NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_SOE_COMMAND_QUEUE,
-            "Failed to register SOE events\n");
-        NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_SOE_BOOTSTRAP,
-            "SOE init failed(2)\n");
-        return status;
+        status = nvswitch_soe_register_event_callbacks(device);
+        if (status != NVL_SUCCESS)
+        {
+            NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_SOE_COMMAND_QUEUE,
+                "Failed to register SOE events\n");
+            NVSWITCH_PRINT_SXID(device, NVSWITCH_ERR_HW_SOE_BOOTSTRAP,
+                "SOE init failed(2)\n");
+            return status;
+        }
+    }
+    else
+    {
+        NVSWITCH_PRINT(device, INFO, "Skipping registering SOE callbacks since TNVL is enabled\n");
     }
 
     // Sanity the command and message queues as a final check
@@ -1363,6 +1440,71 @@ _soeI2CAccess_LS10
     return ret;
 }
 
+/*
+ * @Brief : Send TNVL Pre Lock command to SOE
+ *
+ * @param[in] device
+ */
+NvlStatus
+nvswitch_send_tnvl_prelock_cmd_ls10
+(
+    nvswitch_device *device
+)
+{
+    FLCN            *pFlcn;
+    NvU32            cmdSeqDesc = 0;
+    NV_STATUS        status;
+    RM_FLCN_CMD_SOE  cmd;
+    NVSWITCH_TIMEOUT timeout;
+    RM_SOE_TNVL_CMD_PRE_LOCK_SEQUENCE *pTnvlPreLock;
+    NVSWITCH_GET_BIOS_INFO_PARAMS params = { 0 };
+
+    if (!nvswitch_is_soe_supported(device))
+    {
+        NVSWITCH_PRINT(device, INFO, "%s: SOE is not supported\n",
+                       __FUNCTION__);
+        return -NVL_ERR_NOT_SUPPORTED;
+    }
+
+    status = device->hal.nvswitch_ctrl_get_bios_info(device, &params);
+    if ((status != NVL_SUCCESS) || ((params.version & SOE_VBIOS_VERSION_MASK) <
+            SOE_VBIOS_REVLOCK_TNVL_PRELOCK_COMMAND))
+    {
+        NVSWITCH_PRINT(device, INFO,
+            "%s: Skipping TNVL_CMD_PRE_LOCK_SEQUENCE command to SOE.  Update firmware "
+            "from .%02X to .%02X\n",
+            __FUNCTION__, (NvU32)((params.version & SOE_VBIOS_VERSION_MASK) >> 16), 
+            SOE_VBIOS_REVLOCK_TNVL_PRELOCK_COMMAND);
+        return -NVL_ERR_NOT_SUPPORTED;
+    }
+
+    pFlcn       = device->pSoe->pFlcn;
+
+    nvswitch_os_memset(&cmd, 0, sizeof(cmd));
+    cmd.hdr.unitId = RM_SOE_UNIT_TNVL;
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(TNVL, PRE_LOCK_SEQUENCE);
+
+    pTnvlPreLock = &cmd.cmd.tnvl.preLockSequence;
+    pTnvlPreLock->cmdType = RM_SOE_TNVL_CMD_ISSUE_PRE_LOCK_SEQUENCE;
+
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
+    status = flcnQueueCmdPostBlocking(device, pFlcn,
+                                      (PRM_FLCN_CMD)&cmd,
+                                      NULL,                 // pMsg
+                                      NULL,                 // pPayload
+                                      SOE_RM_CMDQ_LOG_ID,
+                                      &cmdSeqDesc,
+                                      &timeout);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR, "%s: Failed to send PRE_LOCK_SEQUENCE command to SOE, status 0x%x\n", 
+                       __FUNCTION__, status);
+        return -NVL_ERR_GENERIC;
+    }
+
+    return NVL_SUCCESS;
+}
+
 /**
  * @brief   set hal function pointers for functions defined in LR10 (i.e. this file)
  *
