@@ -33,8 +33,13 @@
 #include "vgpu/sdk-structures.h"
 #include "nvRmReg.h"
 
+#include "gpu/spdm/spdm.h"
+#include "gpu/conf_compute/conf_compute.h"
+#include "ctrl/ctrl2080/ctrl2080spdm.h"
 #include "gpu/conf_compute/ccsl.h"
 
+#define IV_NUM_DWORDS 3
+
 NV_STATUS kceConstructEngine_IMPL(OBJGPU *pGpu, KernelCE *pKCe, ENGDESCRIPTOR engDesc)
 {
     NV_ASSERT_OR_RETURN(!RMCFG_FEATURE_PLATFORM_GSP, NV_ERR_NOT_SUPPORTED);
@@ -93,12 +98,76 @@ NvBool kceIsNewMissingEngineRemovalSequenceEnabled_IMPL(OBJGPU *pGpu, KernelCE *
     return NV_TRUE;
 }
 
-#define CE_FIPS_SELF_TEST_DATA_SIZE 16
-#define CE_FIPS_SELF_TEST_AUTH_TAG_SIZE 16
-#define CE_FIPS_SELF_TEST_IV_SIZE 12
+static NV_STATUS
+spdmSendTestCommand
+(
+    OBJGPU       *pGpu,
+    pCcslContext pCcslCtx,
+    NvU8         *text,
+    size_t       textSize,
+    NvU8         *authTag,
+    size_t       authTagSize,
+    NvU32        *iv,
+    size_t       ivSize,
+    NvU8         *encData,
+    size_t       encDataSize,
+    NvU8         isEnc
+)
+{
+    NV_STATUS           status   = NV_OK;
+    RM_API              *pRmApi  = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    RMTIMEOUT           timeout;
+    ConfidentialCompute *pCC   = GPU_GET_CONF_COMPUTE(pGpu);
+    Spdm                *pSpdm = pCC->pSpdm;
 
-NV_STATUS
-kceRunFipsSelfTest
+    NVC56F_CTRL_CMD_GET_KMB_PARAMS             getKmbParams = {0};
+    NV2080_CTRL_INTERNAL_SPDM_PARTITION_PARAMS params = {0};
+    status = pRmApi->Control(pRmApi, pCcslCtx->hClient, pCcslCtx->hChannel,
+            NVC56F_CTRL_CMD_GET_KMB, &getKmbParams,
+            sizeof(getKmbParams));
+
+    NV_ASSERT_OK_OR_RETURN(status);
+
+    if (pGpu == NULL || pSpdm == NULL || !pSpdm->bSessionEstablished)
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    portMemCopy(&params.cmd.ccFipsTest.kmb, sizeof(params.cmd.ccFipsTest.kmb), &getKmbParams.kmb, sizeof(getKmbParams.kmb));
+
+    if (!isEnc)
+    {
+        portMemCopy(&params.cmd.ccFipsTest.kmb.decryptBundle.iv, ivSize, iv, ivSize);
+        portMemCopy(&params.cmd.ccFipsTest.text, textSize, text, textSize);
+        portMemCopy(&params.cmd.ccFipsTest.authTag, authTagSize, authTag, authTagSize);
+    }
+    else
+    {
+        portMemCopy(&params.cmd.ccFipsTest.text, textSize, text, textSize);
+    }
+
+    params.cmd.cmdType          = RM_GSP_SPDM_CMD_ID_FIPS_SELFTEST;
+    params.cmd.ccFipsTest.isEnc = isEnc;
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
+    status = spdmCtrlSpdmPartition(pGpu, &params);
+
+    NV_ASSERT_OK_OR_RETURN(status);
+
+    // Perform basic validation of header returned.
+    status = params.msg.status;
+    
+    NV_ASSERT_OK_OR_RETURN(status);
+
+    if (isEnc) {
+        portMemCopy(encData, encDataSize, params.cmd.ccFipsTest.text, encDataSize);
+        portMemCopy(authTag, authTagSize, params.cmd.ccFipsTest.authTag, authTagSize);
+    }
+
+    return NV_OK;
+}
+
+static NV_STATUS
+kceRunFipsSelfTestDecrypt
 (
     OBJGPU   *pGpu,
     void     *pArg
@@ -116,12 +185,12 @@ kceRunFipsSelfTest
     NV_STATUS          status;
     NV0050_ALLOCATION_PARAMETERS ceUtilsParams = {0};
     CEUTILS_MEMCOPY_PARAMS params = {0};
+    NvU32 Ivl[IV_NUM_DWORDS] = {0};
 
     NvU8 ceTestPlaintext[CE_FIPS_SELF_TEST_DATA_SIZE] = {
         0x2d, 0x71, 0xbc, 0xfa, 0x91, 0x4e, 0x4a, 0xc0,
         0x45, 0xb2, 0xaa, 0x60, 0x95, 0x5f, 0xad, 0x24
     };
-    NvU8 decryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
     NvU8 encryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
     NvU8 dataAuth[CE_FIPS_SELF_TEST_AUTH_TAG_SIZE] = { 0 };
 
@@ -152,6 +221,7 @@ kceRunFipsSelfTest
 
     NV_ASSERT_OK_OR_GOTO(status,
         ccslContextInitViaChannel_HAL(&pCcslCtx, pCeUtils->pChannel->hClient,
+                                      pCeUtils->pChannel->subdeviceId,
                                       pCeUtils->pChannel->channelId),
         failed);
 
@@ -174,6 +244,7 @@ kceRunFipsSelfTest
     TRANSFER_SURFACE srcSurface  = { .pMemDesc = pSrcMemDesc,  .offset = 0 };
     TRANSFER_SURFACE dstSurface  = { .pMemDesc = pDstMemDesc,  .offset = 0 };
     TRANSFER_SURFACE authSurface = { .pMemDesc = pAuthMemDesc, .offset = 0 };
+    TRANSFER_SURFACE ivSurface   = { .pMemDesc = pIvMemDesc,   .offset = 0 };
 
     // Write data to allocations, encrypt using CE, and read back the results
     NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pDstMemDesc, 0, 0), failed);
@@ -198,17 +269,130 @@ kceRunFipsSelfTest
     NV_ASSERT_OK_OR_GOTO(status,
         memmgrMemRead(pMemoryManager, &authSurface, dataAuth, sizeof dataAuth, TRANSFER_FLAGS_NONE), failed);
 
-    // Decrypt using CPU and validate
     NV_ASSERT_OK_OR_GOTO(status,
-        ccslDecrypt_HAL(pCcslCtx, sizeof decryptedData, encryptedData, NULL, NULL, 0, decryptedData, dataAuth),
+        memmgrMemRead(pMemoryManager, &ivSurface, Ivl, sizeof Ivl, TRANSFER_FLAGS_NONE), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, spdmSendTestCommand(pGpu, pCcslCtx, encryptedData, sizeof encryptedData,
+                       dataAuth, sizeof dataAuth, Ivl, sizeof Ivl, NULL, 0, NV_FALSE), failed);
+
+failed:
+    ccslContextClear(pCcslCtx);
+    objDelete(pCeUtils);
+    memdescFree(pSrcMemDesc);
+    memdescDestroy(pSrcMemDesc);
+    memdescFree(pDstMemDesc);
+    memdescDestroy(pDstMemDesc);
+    memdescFree(pAuthMemDesc);
+    memdescDestroy(pAuthMemDesc);
+    memdescFree(pIvMemDesc);
+    memdescDestroy(pIvMemDesc);
+
+    NV_PRINTF(LEVEL_INFO, "Test finished with status 0x%x\n", status);
+
+    return status;
+}
+
+static NV_STATUS
+kceRunFipsSelfTestEncrypt
+(
+    OBJGPU   *pGpu,
+    void     *pArg
+)
+{
+    KernelCE          *pKCe              = pArg;
+    MemoryManager     *pMemoryManager    = GPU_GET_MEMORY_MANAGER(pGpu);
+    KernelMIGManager  *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+    MEMORY_DESCRIPTOR *pSrcMemDesc       = NULL;
+    MEMORY_DESCRIPTOR *pDstMemDesc       = NULL;
+    MEMORY_DESCRIPTOR *pAuthMemDesc      = NULL;
+    MEMORY_DESCRIPTOR *pIvMemDesc        = NULL;
+    CeUtils           *pCeUtils          = NULL;
+    pCcslContext       pCcslCtx          = NULL;
+    NV_STATUS          status;
+    NV0050_ALLOCATION_PARAMETERS ceUtilsParams = {0};
+    CEUTILS_MEMCOPY_PARAMS params = {0};
+    NvU32 Ivl[IV_NUM_DWORDS] = {0};
+
+    NvU8 ceTestPlaintext[CE_FIPS_SELF_TEST_DATA_SIZE] = {
+        0x2d, 0x71, 0xbc, 0xfa, 0x91, 0x4e, 0x4a, 0xc0,
+        0x45, 0xb2, 0xaa, 0x60, 0x95, 0x5f, 0xad, 0x24
+    };
+    NvU8 decryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
+    NvU8 encryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
+    NvU8 dataAuth[CE_FIPS_SELF_TEST_AUTH_TAG_SIZE]  = { 0 };
+
+    NV_ASSERT_OR_RETURN(gpuIsCCFeatureEnabled(pGpu), NV_ERR_NOT_SUPPORTED);
+
+    if (!gpuCheckEngineTable(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)) ||
+        ceIsCeGrce(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)))
+    {
+        // CE doesn't support encryption
+        return NV_OK;
+    }
+
+    if (kmigmgrIsMIGSupported(pGpu, pKernelMIGManager) &&
+        kmigmgrGetStaticInfo(pGpu, pKernelMIGManager) == NULL)
+    {
+        // Wait for KernelMigManager, as it might remap CEs
+        return NV_WARN_MORE_PROCESSING_REQUIRED;
+    }
+
+    NV_PRINTF(LEVEL_INFO, "Running FIPS test for CE%u\n", pKCe->publicID);
+
+    ceUtilsParams.flags |= DRF_DEF(0050_CEUTILS, _FLAGS, _FORCE_CE_ID, _TRUE);
+    ceUtilsParams.flags |= DRF_DEF(0050_CEUTILS, _FLAGS, _CC_SECURE, _TRUE);
+    ceUtilsParams.forceCeId = pKCe->publicID;
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        objCreate(&pCeUtils, pMemoryManager, CeUtils, ENG_GET_GPU(pMemoryManager), NULL, &ceUtilsParams), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        ccslContextInitViaChannel_HAL(&pCcslCtx, pCeUtils->pChannel->hClient,
+                                      pCeUtils->pChannel->subdeviceId,
+                                      pCeUtils->pChannel->channelId),
         failed);
 
-    NV_ASSERT_TRUE_OR_GOTO(status, portMemCmp(decryptedData, ceTestPlaintext, sizeof ceTestPlaintext) == 0,
-        NV_ERR_INVALID_STATE, failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pSrcMemDesc, pGpu, sizeof ceTestPlaintext, 0, NV_TRUE, ADDR_FBMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_ALLOC_FLAGS_PROTECTED), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pSrcMemDesc), failed);
 
-    // Encrypt using CPU
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pDstMemDesc, pGpu, sizeof encryptedData, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pDstMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pAuthMemDesc, pGpu, sizeof dataAuth, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pAuthMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pIvMemDesc, pGpu, sizeof Ivl, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pIvMemDesc), failed);
+
+    TRANSFER_SURFACE srcSurface  = { .pMemDesc = pSrcMemDesc,  .offset = 0 };
+    TRANSFER_SURFACE dstSurface  = { .pMemDesc = pDstMemDesc,  .offset = 0 };
+    TRANSFER_SURFACE authSurface = { .pMemDesc = pAuthMemDesc, .offset = 0 };
+    TRANSFER_SURFACE ivSurface = { .pMemDesc = pIvMemDesc, .offset = 0 };
+
+    // Write data to allocations, encrypt using CE, and read back the results
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pDstMemDesc, 0, 0), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pAuthMemDesc, 0, 0), failed);
     NV_ASSERT_OK_OR_GOTO(status,
-        ccslEncrypt_HAL(pCcslCtx, sizeof ceTestPlaintext, ceTestPlaintext, NULL, 0, encryptedData, dataAuth), failed);
+        memmgrMemWrite(pMemoryManager, &srcSurface, ceTestPlaintext, sizeof ceTestPlaintext, TRANSFER_FLAGS_NONE),
+        failed);
+
+    params.bSecureCopy    = NV_TRUE;
+    params.authTagAddr    = memdescGetPhysAddr(pAuthMemDesc, AT_GPU, 0);
+    params.encryptIvAddr  = memdescGetPhysAddr(pIvMemDesc, AT_GPU, 0);
+    params.pDstMemDesc    = pDstMemDesc;
+    params.dstOffset      = 0;
+    params.pSrcMemDesc    = pSrcMemDesc;
+    params.srcOffset      = 0;
+    params.length         = sizeof ceTestPlaintext;
+    params.bEncrypt       = NV_TRUE;
+    NV_ASSERT_OK_OR_GOTO(status, ceutilsMemcopy(pCeUtils, &params), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, spdmSendTestCommand(pGpu, pCcslCtx, ceTestPlaintext, sizeof ceTestPlaintext, dataAuth,
+                       sizeof dataAuth, Ivl, sizeof Ivl, encryptedData, sizeof encryptedData, NV_TRUE), failed);
 
     // Write data to allocations, decrypt using CE, read back, and validate
     NV_ASSERT_OK_OR_GOTO(status,
@@ -216,6 +400,8 @@ kceRunFipsSelfTest
     NV_ASSERT_OK_OR_GOTO(status,
         memmgrMemWrite(pMemoryManager, &authSurface, dataAuth, sizeof dataAuth, TRANSFER_FLAGS_NONE), failed);
     NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pSrcMemDesc, 0, 0), failed);
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemRead(pMemoryManager, &ivSurface, Ivl, sizeof Ivl, TRANSFER_FLAGS_NONE), failed);
 
     params.pDstMemDesc = pSrcMemDesc;
     params.dstOffset   = 0;
@@ -248,6 +434,22 @@ failed:
     return status;
 }
 
+NV_STATUS
+kceRunFipsSelfTest
+(
+    OBJGPU   *pGpu,
+    void     *pArg
+)
+{
+    NV_STATUS status = NV_OK;
+    status = kceRunFipsSelfTestDecrypt(pGpu, pArg);
+    if (status == NV_OK)
+    {
+        return kceRunFipsSelfTestEncrypt(pGpu, pArg);
+    }
+    return status;
+}
+
 NV_STATUS
 kceStateInitLocked_IMPL
 (
