@@ -30,16 +30,15 @@
 #include "class/clc86fsw.h"
 #include "ctrl/ctrl2080/ctrl2080internal.h"
 #include "nvrm_registry.h"
+#include "kernel/gpu/conf_compute/ccsl.h"
 
 static void initKeyRotationRegistryOverrides(OBJGPU *pGpu, ConfidentialCompute *pConfCompute);
+static void initInternalKeyRotationRegistryOverrides(OBJGPU *pGpu, ConfidentialCompute *pConfCompute);
 static void getKeyPairForKeySpace(NvU32 keySpace, NvBool bKernel, NvU32 *pGlobalH2DKey, NvU32 *pGlobalD2HKey);
 static NV_STATUS triggerKeyRotationByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
 static NV_STATUS calculateEncryptionStatsByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
-static NV_STATUS notifyKeyRotationByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey);
-static NvBool isLowerThresholdCrossed(ConfidentialCompute *pConfCompute, KEY_ROTATION_STATS_INFO *pH2DInfo,
-                                      KEY_ROTATION_STATS_INFO *pD2HInfo);
-static NvBool isUpperThresholdCrossed(ConfidentialCompute *pConfCompute, KEY_ROTATION_STATS_INFO *pH2DInfo,
-                                      KEY_ROTATION_STATS_INFO *pD2HInfo);
+static NvBool isLowerThresholdCrossed(ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
+static NvBool isUpperThresholdCrossed(ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
 static NV_STATUS keyRotationTimeoutCallback(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT *pTmrEvent);
 
 /*!
@@ -60,17 +59,42 @@ confComputeEnableKeyRotationSupport_GH100
         pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED))
     {
         pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_TRUE);
-
-        //
-        // TODO: sbellock default values need to be defined and set separately
-        // for prod flow based on attacker advantage table.
-        //
-        pConfCompute->lowerThreshold.totalBytesEncrypted = NV_U64_MAX;
-        pConfCompute->lowerThreshold.totalEncryptOps = 500;
-        pConfCompute->upperThreshold.totalBytesEncrypted = NV_U64_MAX;
-        pConfCompute->upperThreshold.totalEncryptOps = 1000;
         initKeyRotationRegistryOverrides(pGpu, pConfCompute);
     }
+
+    return NV_OK;
+}
+
+/*!
+ * Conditionally enables key rotation support for keys internal to RM
+ *
+ * @param[in]     pGpu                     : OBJGPU Pointer
+ * @param[in]     pConfCompute             : ConfidentialCompute pointer
+ */
+NV_STATUS
+confComputeEnableInternalKeyRotationSupport_GH100
+(
+    OBJGPU *pGpu,
+    ConfidentialCompute *pConfCompute
+)
+{
+    pConfCompute->keyRotationInternalThreshold = KEY_ROTATION_DEFAULT_INTERNAL_THRESHOLD;
+
+    // Check if we can even support internal key rotation
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_ENABLED) &&
+        pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED))
+    {
+        // TODO CONFCOMP-1740: Default to enabling key rotation when supported
+        // pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED, NV_TRUE);
+        initInternalKeyRotationRegistryOverrides(pGpu, pConfCompute);
+    }
+
+    // If key rotation is disabled, ensure we set values accordingly
+    if (!pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED))
+    {
+        pConfCompute->keyRotationInternalThreshold = 0;
+    }
+
     return NV_OK;
 }
 
@@ -106,6 +130,37 @@ confComputeEnableKeyRotationCallback_GH100
     return NV_OK;
 }
 
+/*!
+ * Returns H2D and D2H keys for given priv level.
+ * For kernel keys, this will only return UVM keys and not the GSP or scrubber keys.
+ *
+ * @param[in]     pGpu                     : OBJGPU Pointer
+ * @param[in]     pConfCompute             : ConfidentialCompute pointer
+ * @param[in]     keySpace                 : keyspace
+ * @param[in]     bKernel                  : is it kernel priv level
+ * @param[out]    pGlobalH2DKey            : h2d key pointer
+ * @param[out]    pGlobalD2HKey            : d2h key pointer
+ */
+void
+confComputeGetKeyPairForKeySpace_GH100
+(
+    OBJGPU *pGpu,
+    ConfidentialCompute *pConfCompute,
+    NvU32 keySpace,
+    NvBool bKernel,
+    NvU32 *pGlobalH2DKey,
+    NvU32 *pGlobalD2HKey
+)
+{
+    NvU32 h2dKey, d2hKey;
+    getKeyPairForKeySpace(keySpace, bKernel, &h2dKey, &d2hKey);
+    if (pGlobalH2DKey != NULL)
+        *pGlobalH2DKey = h2dKey;
+
+    if (pGlobalD2HKey != NULL)
+        *pGlobalD2HKey = d2hKey;
+}
+
  /*!
   * Calculates encryption statistics and triggers key rotation if thresholds are crossed.
   *
@@ -122,27 +177,19 @@ confComputeTriggerKeyRotation_GH100
     NV_STATUS tempStatus, status = NV_OK;
     NvU32 globalD2HKey, globalH2DKey, keySpace;
 
-    if ((pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED)) &&
-        (pConfCompute->keyRotationChannelRefCount > 0))
-    {
-        pConfCompute->keyRotationCallbackCount++;
-        NV_PRINTF(LEVEL_ERROR, "DUMMY KR: COUNT = %d\n", pConfCompute->keyRotationCallbackCount);
-    }
     for (keySpace = 0; keySpace < CC_KEYSPACE_SIZE; keySpace++)
     {
         if (keySpace == CC_KEYSPACE_GSP)
             continue;
 
-        if ((pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED)) &&
-            !(pConfCompute->keyRotationEnableMask & NVBIT(keySpace)))
+        if (!(pConfCompute->keyRotationEnableMask & NVBIT(keySpace)))
         {
             NV_PRINTF(LEVEL_INFO, "Skipping keyspace = %d since mask = 0x%x\n", keySpace, pConfCompute->keyRotationEnableMask);
             continue;
         }
 
-        // calculate kernel channels stats for keyspace
-        if ((!pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED)) ||
-            (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _KERNEL_KEYS, _YES, pConfCompute->keyRotationEnableMask)))
+        // calculate kernel channels stats for keyspace. (this doesn't include scrubber keys)
+        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _KERNEL_KEYS, _YES, pConfCompute->keyRotationEnableMask))
         {
             getKeyPairForKeySpace(keySpace, NV_TRUE, &globalH2DKey, &globalD2HKey);
             tempStatus = triggerKeyRotationByKeyPair(pGpu, pConfCompute, globalH2DKey, globalD2HKey);
@@ -155,8 +202,7 @@ confComputeTriggerKeyRotation_GH100
         }
 
         // calculate user channels stats for keyspace
-        if ((!pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED)) ||
-            (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _USER_KEYS, _YES, pConfCompute->keyRotationEnableMask)))
+        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _USER_KEYS, _YES, pConfCompute->keyRotationEnableMask))
         {
             getKeyPairForKeySpace(keySpace, NV_FALSE, &globalH2DKey, &globalD2HKey);
             tempStatus = triggerKeyRotationByKeyPair(pGpu, pConfCompute, globalH2DKey, globalD2HKey);
@@ -182,28 +228,10 @@ triggerKeyRotationByKeyPair
 {
     KEY_ROTATION_STATUS state;
     NV_ASSERT_OK_OR_RETURN(confComputeGetKeyRotationStatus(pConfCompute, h2dKey, &state));
-    CHANNEL_ITERATOR iter = {0};
-    KernelChannel *pKernelChannel = NULL;
     NvU32 h2dIndex, d2hIndex;
 
-    // we won't need this once we have encryption statistics since unused keys will have stats = 0
-    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
-    {
-        NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
-        if (confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) != NV_OK)
-        {
-            //
-            // If this is the last key and we haven't done KR yet even after crossing upper threshold then
-            // it means there are no channels alive and we need to manually reset our counter
-            //
-            if ((h2dKey == CC_GKEYID_GEN(CC_KEYSPACE_LCE7, CC_LKEYID_LCE_H2D_USER)) &&
-                (pConfCompute->keyRotationCallbackCount > pConfCompute->upperThreshold.totalEncryptOps))
-            {
-                pConfCompute->keyRotationCallbackCount = 1;
-            }
-            return NV_OK;
-        }
-    }
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
 
     //
     // If key rotation is alredy scheduled because we crossed upper threshold or hit timeout
@@ -224,52 +252,109 @@ triggerKeyRotationByKeyPair
 
     NV_ASSERT_OK_OR_RETURN(calculateEncryptionStatsByKeyPair(pGpu, pConfCompute, h2dKey, d2hKey));
 
-    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
-    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
-
-    if (isUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex],
-                                &pConfCompute->aggregateStats[d2hIndex]))
+    if (isUpperThresholdCrossed(pConfCompute, h2dKey, d2hKey))
     {
         NV_PRINTF(LEVEL_ERROR, "Crossed UPPER threshold for key = 0x%x\n", h2dKey);
         NV_ASSERT_OK_OR_RETURN(confComputeSetKeyRotationStatus(pConfCompute, h2dKey, KEY_ROTATION_STATUS_FAILED_THRESHOLD));
-        NV_ASSERT_OK_OR_RETURN(confComputeScheduleKeyRotationWorkItem(pGpu, pConfCompute, h2dKey, d2hKey));
+        NV_ASSERT_OK_OR_RETURN(confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE));
     }
-    else if (isLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex],
-                                     &pConfCompute->aggregateStats[d2hIndex]))
+    else if (isLowerThresholdCrossed(pConfCompute, h2dKey, d2hKey))
     {
         NV_PRINTF(LEVEL_INFO, "Crossed LOWER threshold for key = 0x%x\n", h2dKey);
         if (state == KEY_ROTATION_STATUS_IDLE)
         {
             NV_ASSERT_OK_OR_RETURN(confComputeSetKeyRotationStatus(pConfCompute, h2dKey, KEY_ROTATION_STATUS_PENDING));
 
-            //
-            // Start the timeout timer once lower threshold is crossed.
-            //
-            // If timer is not already created then create it now. Else, just schedule a callback.
-            // make sure callback is canceled if we schedule the KR task (after crossing lower or upper threshold)
-            // make sure all these timer events are deleted as part of RM shutdown
-            //
-            OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
-            if (pConfCompute->ppKeyRotationTimer[h2dIndex] == NULL)
+            // Schedule first callback. Timeouts are not enforced on kernel channels (UVM).
+            if (!confComputeGlobalKeyIsUvmKey_HAL(pConfCompute, h2dKey))
             {
-                NvU32 *pH2DKey = portMemAllocNonPaged(sizeof(NvU32));
-                *pH2DKey = h2dKey;
-                NV_ASSERT_OK_OR_RETURN(tmrEventCreate(pTmr, &pConfCompute->ppKeyRotationTimer[h2dIndex],
-                                                      keyRotationTimeoutCallback, (void*)pH2DKey, TMR_FLAGS_NONE));
+                //
+                // Start the timeout timer once lower threshold is crossed.
+                //
+                // If timer is not already created then create it now. Else, just schedule a callback.
+                // make sure callback is canceled if we schedule the KR task (after crossing lower or upper threshold)
+                // make sure all these timer events are deleted as part of RM shutdown
+                //
+                OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+                if (pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer == NULL)
+                {
+                    NvU32 *pH2DKey = portMemAllocNonPaged(sizeof(NvU32));
+                    *pH2DKey = h2dKey;
+                    NV_ASSERT_OK_OR_RETURN(tmrEventCreate(pTmr, &pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer,
+                                                          keyRotationTimeoutCallback, (void*)pH2DKey, TMR_FLAGS_NONE));
+                }
+                if (confComputeIsUvmKeyRotationPending(pGpu, pConfCompute))
+                {
+                    // If there is any UVM KR pending do not schedule the timer for timeout
+                    NV_ASSERT_OK_OR_RETURN(confComputeStopKeyRotationTimer(pGpu, pConfCompute, h2dKey));
+                }
+                else
+                {
+                    // no kernel KR pending, start/schedule timer for timeout
+                    NV_ASSERT_OK_OR_RETURN(tmrEventScheduleRelSec(pTmr,
+                                                                  pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer,
+                                                                  pConfCompute->keyRotationTimeout));
+                }
+            }
+            else
+            {
+                // if this is a kernel KR, check if there are any pending user KRs
+                NvU32 userH2DKey;
+                KEY_ROTATION_STATUS userKRStatus;
+                NvU32 keySpace;
+                for (keySpace = 0; keySpace < CC_KEYSPACE_SIZE; keySpace++)
+                {
+                    if (keySpace == CC_KEYSPACE_GSP)
+                        continue;
+
+                    confComputeGetKeyPairForKeySpace_HAL(pGpu, pConfCompute, keySpace, NV_FALSE, &userH2DKey, NULL);
+                    NV_ASSERT_OK_OR_RETURN(confComputeGetKeyRotationStatus(pConfCompute, userH2DKey, &userKRStatus));
+                    if (userKRStatus == KEY_ROTATION_STATUS_PENDING)
+                    {
+                        NV_PRINTF(LEVEL_INFO, "User key rotation pending on key 0x%x\n", userH2DKey);
+                        NV_ASSERT_OK_OR_RETURN(confComputeStopKeyRotationTimer(pGpu, pConfCompute, userH2DKey));
+                    }
+                }
             }
 
-            //
-            // Schedule first callback.
-            // TODO sbellock: don't use hardcoded 12.5 sec vaule
-            //
-            NV_ASSERT_OK_OR_RETURN(tmrEventScheduleRelSec(pTmr, pConfCompute->ppKeyRotationTimer[h2dIndex], 12.5));
-
             //
             // Notify clients of pending KR
-            // We can't schedule a workitem for this since it may get scheduled too late and
+            // We can't schedule a workitem for notifying since it may get scheduled too late and
             // we might have already crossed the upper threshold by then.
             //
-            NV_ASSERT_OK_OR_RETURN(notifyKeyRotationByKeyPair(pGpu, pConfCompute, h2dKey));
+            {
+                CHANNEL_ITERATOR iter = {0};
+                KernelChannel *pKernelChannel = NULL;
+
+                // notify all channels
+                NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
+                while(confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) == NV_OK)
+                {
+                    NV_ASSERT_OK_OR_RETURN(kchannelUpdateNotifierMem(pKernelChannel, NV_CHANNELGPFIFO_NOTIFICATION_TYPE_KEY_ROTATION_STATUS,
+                                                                     pConfCompute->keyRotationCount[h2dIndex], 0, (NvU32)KEY_ROTATION_STATUS_PENDING));
+                    NV_PRINTF(LEVEL_INFO, "chid 0x%x has pending key rotation, writing notifier with val 0x%x\n",
+                    kchannelGetDebugTag(pKernelChannel), (NvU32)KEY_ROTATION_STATUS_PENDING);
+
+                    // send events to clients if registered
+                    kchannelNotifyEvent(pKernelChannel, NVC86F_NOTIFIERS_KEY_ROTATION, pConfCompute->keyRotationCount[h2dIndex],
+                                        KEY_ROTATION_STATUS_PENDING, NULL, 0);
+                }
+            }
+        }
+
+        //
+        // There can be a scenario where last channel on key gets freed without lower threshold being
+        // crossed but when encrypt stats are computed in the next callback lower threshold is crossed here.
+        // In this case, schedule key rotation.
+        //
+        {
+            CHANNEL_ITERATOR iter = {0};
+            KernelChannel *pKernelChannel = NULL;
+            NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
+            if (confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) != NV_OK)
+            {
+                NV_ASSERT_OK_OR_RETURN(confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE));
+            }
         }
     }
     return NV_OK;
@@ -285,10 +370,12 @@ keyRotationTimeoutCallback
 {
     ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
     NvU32 h2dKey, d2hKey;
+    NV_ASSERT_OR_RETURN(pEvent->pUserData != NULL, NV_ERR_INVALID_STATE);
     NvU32 key = *(NvU32*)pEvent->pUserData;
     confComputeGetKeyPairByKey(pConfCompute, key, &h2dKey, &d2hKey);
     NV_ASSERT_OK_OR_RETURN(confComputeSetKeyRotationStatus(pConfCompute, h2dKey, KEY_ROTATION_STATUS_FAILED_TIMEOUT));
-    return confComputeScheduleKeyRotationWorkItem(pGpu, pConfCompute, h2dKey, d2hKey);
+    NV_PRINTF(LEVEL_INFO, "Hit timeout on key 0x%x, triggering KR\n", h2dKey);
+    return confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE);
 }
 
 static NV_STATUS
@@ -355,8 +442,8 @@ calculateEncryptionStatsByKeyPair
     {
         NV_PRINTF(LEVEL_INFO, "Aggregate stats for h2dKey 0x%x and d2hKey 0x%x\n", h2dKey, d2hKey);
         NV_PRINTF(LEVEL_INFO, "Total h2d bytes encrypted  = 0x%llx\n", pConfCompute->aggregateStats[h2dIndex].totalBytesEncrypted);
-        NV_PRINTF(LEVEL_INFO, "Total d2h bytes encrypted  = 0x%llx\n", pConfCompute->aggregateStats[h2dIndex].totalEncryptOps);
-        NV_PRINTF(LEVEL_INFO, "Total h2d encrypt ops  = 0x%llx\n", pConfCompute->aggregateStats[d2hIndex].totalBytesEncrypted);
+        NV_PRINTF(LEVEL_INFO, "Total d2h bytes encrypted  = 0x%llx\n", pConfCompute->aggregateStats[d2hIndex].totalBytesEncrypted);
+        NV_PRINTF(LEVEL_INFO, "Total h2d encrypt ops  = 0x%llx\n", pConfCompute->aggregateStats[h2dIndex].totalEncryptOps);
         NV_PRINTF(LEVEL_INFO, "Total d2h encrypt ops  = 0x%llx\n", pConfCompute->aggregateStats[d2hIndex].totalEncryptOps);
     }
     return NV_OK;
@@ -366,38 +453,33 @@ static NvBool
 isUpperThresholdCrossed
 (
     ConfidentialCompute *pConfCompute,
-    KEY_ROTATION_STATS_INFO *pH2DInfo,
-    KEY_ROTATION_STATS_INFO *pD2HInfo
+    NvU32 h2dKey,
+    NvU32 d2hKey
 )
 {
-    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
-    {
-        return (pConfCompute->keyRotationCallbackCount > pConfCompute->upperThreshold.totalEncryptOps);
-    }
-    else
-    {
-        return (confComputeIsUpperThresholdCrossed(pConfCompute, pH2DInfo) ||
-                confComputeIsUpperThresholdCrossed(pConfCompute, pD2HInfo));
-    }
+    NvU32 h2dIndex, d2hIndex;
+
+    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
+    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
+
+    return (confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
+            confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
 }
 
 static NvBool
 isLowerThresholdCrossed
 (
     ConfidentialCompute *pConfCompute,
-    KEY_ROTATION_STATS_INFO *pH2DInfo,
-    KEY_ROTATION_STATS_INFO *pD2HInfo
+    NvU32 h2dKey,
+    NvU32 d2hKey
 )
 {
-    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
-    {
-        return (pConfCompute->keyRotationCallbackCount > pConfCompute->lowerThreshold.totalEncryptOps);
-    }
-    else
-    {
-        return (confComputeIsLowerThresholdCrossed(pConfCompute, pH2DInfo) ||
-                confComputeIsLowerThresholdCrossed(pConfCompute, pD2HInfo));
-    }
+    NvU32 h2dIndex, d2hIndex;
+    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
+    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
+
+    return (confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
+            confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
 }
 
 static void
@@ -434,46 +516,6 @@ getKeyPairForKeySpace(NvU32 keySpace, NvBool bKernel, NvU32 *pGlobalH2DKey, NvU3
     *pGlobalD2HKey = CC_GKEYID_GEN(keySpace, localD2HKey);
 }
 
-static NV_STATUS
-notifyKeyRotationByKeyPair
-(
-    OBJGPU *pGpu,
-    ConfidentialCompute *pConfCompute,
-    NvU32 h2dKey
-)
-{
-    KEY_ROTATION_STATUS status;
-    CHANNEL_ITERATOR iter = {0};
-    KernelChannel *pKernelChannel = NULL;
-    NvU32 notifyStatus = 0;
-    NV_ASSERT_OK_OR_RETURN(confComputeGetKeyRotationStatus(pConfCompute, h2dKey, &status));
-    //
-    // We expect this work item to be called soon after RM detects lower threshold is
-    // crossed and schedules this.
-    //
-    NV_ASSERT_OR_RETURN(status == KEY_ROTATION_STATUS_PENDING, NV_ERR_INVALID_STATE);
-
-	// notify all channels
-	NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
-	while(confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) == NV_OK)
-	{
-		// update notifier memory
-		notifyStatus =
-			FLD_SET_DRF(_CHANNELGPFIFO, _NOTIFICATION_STATUS, _IN_PROGRESS, _TRUE, notifyStatus);
-
-		notifyStatus =
-			FLD_SET_DRF_NUM(_CHANNELGPFIFO, _NOTIFICATION_STATUS, _VALUE, status, notifyStatus);
-
-		NV_ASSERT_OK_OR_RETURN(kchannelUpdateNotifierMem(pKernelChannel, NV_CHANNELGPFIFO_NOTIFICATION_TYPE_KEY_ROTATION_STATUS,
-											             0, 0, notifyStatus));
-		NV_PRINTF(LEVEL_INFO, "chid 0x%x has pending key rotation, writing notifier with val 0x%x\n", kchannelGetDebugTag(pKernelChannel), (NvU32)notifyStatus);
-
-		// send events to clients if registered
-		kchannelNotifyEvent(pKernelChannel, NVC86F_NOTIFIERS_KEY_ROTATION, 0, status, NULL, 0);
-	}
-    return NV_OK;
-}
-
 static void
 initKeyRotationRegistryOverrides
 (
@@ -481,48 +523,134 @@ initKeyRotationRegistryOverrides
     ConfidentialCompute *pConfCompute
 )
 {
-    //
-    // Temp CONFCOMP-984: This will be removed once all RM clients support
-    // key rotation by default.
-    //
     if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED))
     {
         NvU32 data;
-        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, &data) == NV_OK)
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_THRESHOLD_DELTA, &data) == NV_OK)
         {
-            if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _ENABLED, _YES, data))
+            if (data == 0)
             {
-                NV_PRINTF(LEVEL_INFO, "Confidential Compute dummy key rotation enabled via regkey override.\n");
-                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED, NV_TRUE);
+                NV_PRINTF(LEVEL_ERROR, "Illegal value for RmKeyRotationThresholdDelta.\n");
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of threshold delta.\n");
             }
-            else if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _ENABLED, _NO, data))
+            else
             {
-                NV_PRINTF(LEVEL_INFO, "Confidential Compute dummy key rotation disabled via regkey override.\n");
-                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED, NV_FALSE);
-            }
-            if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
-            {
-                pConfCompute->keyRotationEnableMask = data;
-
-                //
-                // Set lower and upper thresholds to default values
-                // this will go away once we stop supporting dummy KR
-                //
-                pConfCompute->lowerThreshold.totalBytesEncrypted = NV_U64_MAX;
-                pConfCompute->lowerThreshold.totalEncryptOps = KEY_ROTATION_LOWER_THRESHOLD;
-                pConfCompute->upperThreshold.totalBytesEncrypted = NV_U64_MAX;
-                pConfCompute->upperThreshold.totalEncryptOps = KEY_ROTATION_UPPER_THRESHOLD;
-
-                if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION_LOWER_THRESHOLD, &data) == NV_OK)
-                {
-                    pConfCompute->lowerThreshold.totalEncryptOps = data;
-                }
-
-                if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION_UPPER_THRESHOLD, &data) == NV_OK)
-                {
-                    pConfCompute->upperThreshold.totalEncryptOps = data;
-                }
+                NV_PRINTF(LEVEL_INFO, "Setting key rotation threshold delta to %u.\n", data);
+                pConfCompute->keyRotationThresholdDelta = data;
+                NV_ASSERT_OK(confComputeSetKeyRotationThreshold(pConfCompute, pConfCompute->attackerAdvantage));
             }
         }
+        else if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_LOWER_THRESHOLD, &data) == NV_OK)
+        {
+            const NvU32 lowerThreshold = data;
+
+            if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_UPPER_THRESHOLD, &data) == NV_OK)
+            {
+                const NvU32 upperThreshold = data;
+
+                if (upperThreshold > lowerThreshold)
+                {
+                    NV_PRINTF(LEVEL_INFO, "Setting key rotation lower threshold to %u and upper threshold to %u.\n",
+                              lowerThreshold, upperThreshold);
+                    pConfCompute->keyRotationUpperThreshold = upperThreshold;
+                    pConfCompute->keyRotationLowerThreshold = lowerThreshold;
+                }
+                else
+                {
+                    NV_PRINTF(LEVEL_ERROR, "RmKeyRotationUpperThreshold must be greater than RmKeyRotationLowerThreshold.\n");
+                    NV_PRINTF(LEVEL_ERROR, "Cancelling override of upper and lower key rotation thresholds.\n");
+                }
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "RmKeyRotationUpperThreshold must be set if RmKeyRotationLowerThreshold is set.\n");
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of upper and lower key rotation thresholds.\n");
+            }
+        }
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_TIMEOUT_IN_SEC, &data) == NV_OK)
+        {
+            const NvU32 minTimeout = 2;
+
+            if (data >= minTimeout)
+            {
+                NV_PRINTF(LEVEL_INFO, "Setting key rotation user-mode timeout to %u seconds.\n", data);
+                pConfCompute->keyRotationTimeout = data;
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "Key rotation user-mode timeout must be greater than or equal to %u.\n", minTimeout);
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of user-mode timeout.\n");
+            }
+        }
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION, &data) == NV_OK)
+        {
+            if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _ENABLED, _YES, data))
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation enabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_TRUE);
+                pConfCompute->keyRotationEnableMask = data;
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation disabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_FALSE);
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_FALSE);
+            }
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation is disabled.\n");
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_FALSE);
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_FALSE);
+        }
     }
-}
\ No newline at end of file
+}
+
+static void
+initInternalKeyRotationRegistryOverrides
+(
+    OBJGPU              *pGpu,
+    ConfidentialCompute *pConfCompute
+)
+{
+    NvU32 data = 0;
+
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_MULTI_GPU_PROTECTED_PCIE_MODE_ENABLED))
+    {
+        NV_PRINTF(LEVEL_INFO, "RM internal key rotation not supported for protected PCIe!\n");
+        pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED, NV_FALSE);
+    }
+    else if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION, &data) == NV_OK)
+    {
+        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _INTERNAL_KEYS, _YES, data))
+        {
+            NV_PRINTF(LEVEL_ERROR, "Enabling RM internal keys for Key Rotation by regkey override!\n");
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED, NV_TRUE);
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_ERROR, "Disabling RM internal keys for Key Rotation by regkey override!\n");
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED, NV_FALSE);
+        }
+    }
+
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_INTERNAL_KEY_ROTATION_ENABLED) &&
+        (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_INTERNAL_THRESHOLD, &data) == NV_OK))
+    {
+        const NvU32 internalThreshold = data;
+        if (internalThreshold < KEY_ROTATION_MINIMUM_INTERNAL_THRESHOLD)
+        {
+            NV_PRINTF(LEVEL_ERROR, "RmKeyRotationInternalThreshold must be higher than minimum of %u!\n",
+                        KEY_ROTATION_MINIMUM_INTERNAL_THRESHOLD);
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "Setting internal key rotation threshold to %u.\n",
+                        internalThreshold);
+            pConfCompute->keyRotationInternalThreshold = internalThreshold;
+        }
+    }
+}
