@@ -27,15 +27,18 @@
 
 #include "rmconfig.h"
 #include "gpu/conf_compute/conf_compute.h"
+#include "spdm/rmspdmtransport.h"
 #include "gpu/fsp/kern_fsp.h"
 #include "gpu/gsp/kernel_gsp.h"
 #include "gpu/mem_sys/kern_mem_sys.h"
 #include "gsp/gspifpub.h"
 #include "vgpu/rpc.h"
+#include "os/os.h"
 
 #include "published/hopper/gh100/dev_falcon_v4.h"
 #include "published/hopper/gh100/dev_gsp.h"
 #include "published/hopper/gh100/dev_riscv_pri.h"
+#include "published/hopper/gh100/dev_vm.h"
 
 #define RISCV_BR_ADDR_ALIGNMENT                 (8)
 
@@ -299,8 +302,6 @@ kgspCalculateFbLayout_GH100
     NV_ASSERT_OR_RETURN(pKernelGsp->gspRmBootUcodeSize != 0, NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pRiscvDesc != NULL, NV_ERR_INVALID_STATE);
 
-    portMemSet(pWprMeta, 0, sizeof *pWprMeta);
-
     //
     // We send this to FSP as the size to reserve above FRTS.
     // The actual offset gets filled in by ACR ucode when it sets up WPR2.
@@ -412,30 +413,203 @@ kgspSetupGspFmcArgs_GH100
     pGspFmcBootParams->gspRmParams.bootArgsOffset = memdescGetPhysAddr(pKernelGsp->pLibosInitArgumentsDescriptor, AT_GPU, 0);
     pGspFmcBootParams->gspRmParams.target = _kgspMemdescToDmaTarget(pKernelGsp->pLibosInitArgumentsDescriptor);
 
+    if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_SPDM_ENABLED))
+    {
+        NV_STATUS  status = NV_OK;
+        Spdm      *pSpdm  = pCC->pSpdm;
+
+        // If SPDM is NULL, we failed to initialize
+        if (pCC->pSpdm == NULL)
+        {
+            return NV_ERR_INVALID_STATE;
+        }
+
+        // Perform required pre-GSP-RM boot setup that could not be done on Spdm object creation.
+        status = spdmSetupCommunicationBuffers(pGpu, pSpdm);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Failure when initializing SPDM messaging infrastructure. Status:0x%x\n", status);
+            return status;
+        }
+
+        // Target will always be GSP_DMA_TARGET_COHERENT_SYSTEM
+        pGspFmcBootParams->gspSpdmParams.target               = GSP_DMA_TARGET_COHERENT_SYSTEM;
+        pGspFmcBootParams->gspSpdmParams.payloadBufferOffset  = memdescGetPhysAddr(pSpdm->pPayloadBufferMemDesc,
+                                                                                   AT_GPU, 0);
+        pGspFmcBootParams->gspSpdmParams.payloadBufferSize    = pSpdm->payloadBufferSize;
+    }
+
     return NV_OK;
 }
 
 /*!
- * Determine if lockdown is released.
+ * Determine if PRIV lockdown is released or the FMC has encountered an error.
  */
 static NvBool
-_kgspIsLockdownReleased
+_kgspLockdownReleasedOrFmcError
 (
     OBJGPU  *pGpu,
     void    *pVoid
 )
 {
     KernelGsp *pKernelGsp = reinterpretCast(pVoid, KernelGsp *);
-    NvU32 reg;
+    KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
+    NvU32 hwcfg2, mailbox0;
 
-    reg = kflcnRegRead_HAL(pGpu, staticCast(pKernelGsp, KernelFalcon), NV_PFALCON_FALCON_HWCFG2);
+    //
+    // If lockdown has not been released, check NV_PGSP_FALCON_MAILBOX0, where the GSP-FMC
+    // (namely ACR) logs error codes during boot. GSP-FMC reported errors are always fatal,
+    // so there's no reason to continue polling for lockdown release.
+    //
+    mailbox0 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0);
+    hwcfg2 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_HWCFG2);
 
-    return FLD_TEST_DRF(_PFALCON, _FALCON_HWCFG2, _RISCV_BR_PRIV_LOCKDOWN,
-                        _UNLOCK, reg);
+    return (FLD_TEST_DRF(_PFALCON, _FALCON_HWCFG2, _RISCV_BR_PRIV_LOCKDOWN, _UNLOCK, hwcfg2) ||
+            (mailbox0 != 0));
 }
 
+/*!
+ * Determine if SPDM partition has booted or the FMC has encountered an error.
+ */
+static NvBool
+_kgspSpdmBootedOrFmcError
+(
+    OBJGPU  *pGpu,
+    void    *pVoid
+)
+{
+    //
+    // The GSP-FMC (namely ACR) logs error codes to NV_PGSP_FALCON_MAILBOX0 during boot.
+    // During normal boot, the mailboxes should be 0 (cleared by the GSP-FMC ucode at the
+    // start), so consider any non-zero value here a reason to stop polling.
+    //
+    return (kflcnRegRead_HAL(pGpu, (KernelFalcon *)pVoid, NV_PFALCON_FALCON_MAILBOX0) != 0);
+}
 
+static NvBool
+_kgspFalconMailbox0Cleared
+(
+    OBJGPU  *pGpu,
+    void    *pVoid
+)
+{
+    return (kflcnRegRead_HAL(pGpu, (KernelFalcon *)pVoid, NV_PFALCON_FALCON_MAILBOX0) == 0);
+}
 
+/*!
+ * Establish session with SPDM Responder on GSP in Confidential Compute scenario.
+ * GSP-RM boot is blocked until session establishment completes.
+ *
+ * NOTE: This function currently requires the API lock (for at least the async init
+ * RPCs and libspdm synchronization) and as such does NOT support parallel init.
+ */
+static NV_STATUS
+_kgspEstablishSpdmSession
+(
+    OBJGPU              *pGpu,
+    KernelGsp           *pKernelGsp,
+    ConfidentialCompute *pConfCompute
+)
+{
+    NV_STATUS     status        = NV_OK;
+    KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
+
+    // Ensure SPDM Responder has booted before attempting to establish session.
+    status = gpuTimeoutCondWait(pGpu, _kgspSpdmBootedOrFmcError, pKernelFalcon, NULL);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Timeout waiting for SPDM Responder to initialize!\n");
+        goto exit;
+    }
+
+    //
+    // Check if the terminating condition of the above wait was that SPDM has booted, or
+    // if the FMC produced an error code in the mailbox.
+    //
+    NvU32 mailbox0 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0);
+    if (mailbox0 != NV_SPDM_PARTITION_BOOT_SUCCESS)
+    {
+        NV_PRINTF(LEVEL_ERROR, "GSP-FMC reported an error prior to SPDM boot: 0x%x\n",
+                  mailbox0);
+        status = NV_ERR_NOT_READY;
+        goto exit;
+    }
+
+    status = confComputeEstablishSpdmSessionAndKeys_HAL(pGpu, pConfCompute);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "SPDM handshake with Responder failed.\n");
+        goto exit;
+    }
+
+    //
+    // Now we can send the async init messages that will be run before OBJGPU is created.
+    // SPDM will not continue with boot until we send ack after sending these RPCs, else
+    // we create a race condition where GSP-RM may skip these RPCs if not sent in time.
+    //
+    status = kgspQueueAsyncInitRpcs(pGpu, pKernelGsp);
+    if (status != NV_OK)
+    {
+        goto exit;
+    }
+
+    // Tell SPDM that it can continue with boot
+    kflcnRegWrite_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0, NV_SPDM_REQUESTER_SECRETS_DERIVED);
+
+    //
+    // Wait for SPDM to restore the original mailbox value, which should have been 0 prior
+    // to SPDM boot starting. This is needed because the subsequent lockdown release check
+    // will also consider any non-zero value in mailbox0 to indicate an FMC error code.
+    //
+    status = gpuTimeoutCondWait(pGpu, _kgspFalconMailbox0Cleared, (void *)pKernelFalcon, NULL);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Timeout waiting for SPDM to proceed with boot!\n");
+        goto exit;
+    }
+
+exit:
+    if (status != NV_OK)
+    {
+        KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+
+        NV_PRINTF(LEVEL_ERROR, "Failed to establish session with SPDM Responder!\n");
+        if (pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_GSP_MODE_GSPRM))
+        {
+            kfspDumpDebugState_HAL(pGpu, pKernelFsp);
+        }
+        NV_PRINTF(LEVEL_ERROR, "NV_PGSP_FALCON_MAILBOX0 = 0x%x\n",
+            kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0));
+        NV_PRINTF(LEVEL_ERROR, "NV_PGSP_FALCON_MAILBOX1 = 0x%x\n",
+            kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1));
+    }
+
+    return status;
+}
+
+/*!
+ * Determine if GSP has performed CC partition cleanup.
+ * Success or failure will be determined by the value in Mailbox1
+ */
+static NvBool
+_kgspHasCcCleanupFinished
+(
+    OBJGPU  *pGpu,
+    void    *pVoid
+)
+{
+    NvU32         ccCleanupStatus = 0;
+    KernelFalcon *pKernelFalcon   = (KernelFalcon *)pVoid;
+
+    ccCleanupStatus = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1);
+
+    //
+    // To avoid a timing issue and issues with pre-existing values in Mailbox1
+    // we check for explicit success or failure statuses.
+    //
+    return (ccCleanupStatus == NV_SPDM_SECRET_TEARDOWN_SUCCESS ||
+            ccCleanupStatus == NV_SPDM_SECRET_TEARDOWN_FAILURE);
+}
 
 static void
 _kgspBootstrapGspFmc_GH100
@@ -497,18 +671,64 @@ _kgspBootstrapGspFmc_GH100
                            DRF_DEF(_PRISCV_RISCV, _CPUCTL, _STARTCPU, _TRUE));
 }
 
+/*!
+ * Prepare to boot GSP-RM
+ *
+ * This routine handles the prequesites to booting GSP-RM that requires the API LOCK:
+ *   - Clear ECC errors
+ *   - Prepare GSP-FMC arguments
+ *   - Prepare FSP boot commands
+ *
+ * @param[in]   pGpu            GPU object pointer
+ * @param[in]   pKernelGsp      GSP object pointer
+ * @param[in]   pGspFw          GSP_FIRMWARE image pointer
+ *
+ * @return NV_OK if GSP-RM RISCV preparation to boot was successful.
+ *         Appropriate NV_ERR_xxx value otherwise.
+ */
+NV_STATUS
+kgspPrepareForBootstrap_GH100
+(
+    OBJGPU         *pGpu,
+    KernelGsp      *pKernelGsp,
+    GSP_FIRMWARE   *pGspFw
+)
+{
+    KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+
+    // Only for GSP client builds
+    if (!IS_GSP_CLIENT(pGpu))
+    {
+        NV_PRINTF(LEVEL_ERROR, "IS_GSP_CLIENT is not set.\n");
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    // Setup the descriptors that GSP-FMC needs to boot GSP-RM
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw));
+
+    if (pKernelFsp != NULL && !pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_GSPFMC))
+    {
+        pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_GSP_MODE_GSPRM, NV_TRUE);
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            kfspPrepareBootCommands_HAL(pGpu, pKernelFsp));
+    }
+
+    return NV_OK;
+}
+
 /*!
  * Boot GSP-RM.
  *
  * This routine handles the following:
- *   - prepares RISCV core to run GSP-RM
- *   - prepares libos initialization args
- *   - prepares GSP-RM initialization message
- *   - starts the RISCV core and passes control to boot binary image
+ *   - sends FSP boot commands
  *   - waits for GSP-RM to complete initialization
  *
  * Note that this routine is based on flcnBootstrapRiscvOS_GA102().
  *
+ * Note that this routine can be called without the API lock for
+ * parllel initialization.
+ *
  * @param[in]   pGpu            GPU object pointer
  * @param[in]   pKernelGsp      GSP object pointer
  * @param[in]   pGspFw          GSP_FIRMWARE image pointer
@@ -517,7 +737,7 @@ _kgspBootstrapGspFmc_GH100
  *         Appropriate NV_ERR_xxx value otherwise.
  */
 NV_STATUS
-kgspBootstrapRiscvOSEarly_GH100
+kgspBootstrap_GH100
 (
     OBJGPU         *pGpu,
     KernelGsp      *pKernelGsp,
@@ -525,57 +745,15 @@ kgspBootstrapRiscvOSEarly_GH100
 )
 {
     KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
-    KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
-    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
-    NV_STATUS     status        = NV_OK;
-
-    // Only for GSP client builds
-    if (!IS_GSP_CLIENT(pGpu))
-    {
-        NV_PRINTF(LEVEL_ERROR, "IS_GSP_CLIENT is not set.\n");
-        return NV_ERR_NOT_SUPPORTED;
-    }
-
-    // Clear ECC errors before attempting to load GSP
-    status = kmemsysClearEccCounts_HAL(pGpu, pKernelMemorySystem);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Issue clearing ECC counts! Status:0x%x\n", status);
-    }
-
-    // Setup the descriptors that GSP-FMC needs to boot GSP-RM
-    NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-            kgspSetupGspFmcArgs_HAL(pGpu, pKernelGsp, pGspFw), exit);
-
-    kgspSetupLibosInitArgs(pGpu, pKernelGsp);
-
-    // Fill in the GSP-RM message queue init parameters
-    kgspPopulateGspRmInitArgs(pGpu, pKernelGsp, NULL);
-
-    //
-    // Stuff the message queue with async init messages that will be run
-    // before OBJGPU is created.
-    //
-    NV_RM_RPC_GSP_SET_SYSTEM_INFO(pGpu, status);
-    if (status != NV_OK)
-    {
-        NV_ASSERT_OK_FAILED("NV_RM_RPC_GSP_SET_SYSTEM_INFO", status);
-        goto exit;
-    }
-
-    NV_RM_RPC_SET_REGISTRY(pGpu, status);
-    if (status != NV_OK)
-    {
-        NV_ASSERT_OK_FAILED("NV_RM_RPC_SET_REGISTRY", status);
-        goto exit;
-    }
+    KernelFsp    *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+    NV_STATUS     status = NV_OK;
+    NvU32         mailbox0;
+    ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
 
     if (pKernelFsp != NULL && !pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_GSPFMC))
     {
         NV_PRINTF(LEVEL_NOTICE, "Starting to boot GSP via FSP.\n");
-        pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_GSP_MODE_GSPRM, NV_TRUE);
-        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                kfspSendBootCommands_HAL(pGpu, pKernelFsp), exit);
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, kfspSendBootCommands_HAL(pGpu, pKernelFsp));
     }
     else
     {
@@ -598,12 +776,24 @@ kgspBootstrapRiscvOSEarly_GH100
                 kfspDumpDebugState_HAL(pGpu, pKernelFsp);
             }
 
-            goto exit;
+            return status;
         }
     }
 
-    // Wait for lockdown to be released.
-    status = gpuTimeoutCondWait(pGpu, _kgspIsLockdownReleased, pKernelGsp, NULL);
+    //
+    // In Confidential Compute, SPDM session will be established before GSP-RM boots.
+    // Wait until after target mask is released by ACR to minimize busy wait time.
+    // NOTE: This is incompatible with parallel initialization. See function
+    // description for more details.
+    //
+    if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED))
+    {
+        NV_ASSERT_OR_RETURN(rmapiLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
+        NV_ASSERT_OK_OR_RETURN(_kgspEstablishSpdmSession(pGpu, pKernelGsp, pCC));
+    }
+
+    // Wait for lockdown to be released or the FMC to report an error
+    status = gpuTimeoutCondWait(pGpu, _kgspLockdownReleasedOrFmcError, pKernelGsp, NULL);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Timeout waiting for lockdown release. It's also "
@@ -619,7 +809,13 @@ kgspBootstrapRiscvOSEarly_GH100
                   kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0));
         NV_PRINTF(LEVEL_ERROR, "NV_PGSP_FALCON_MAILBOX1 = 0x%x\n",
                   kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1));
-        goto exit;
+        return status;
+    }
+    else if ((mailbox0 = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX0)) != 0)
+    {
+        NV_PRINTF(LEVEL_ERROR, "GSP-FMC reported an error while attempting to boot GSP: 0x%x\n",
+                  mailbox0);
+        return NV_ERR_NOT_READY;
     }
 
     // Start polling for libos logs now that lockdown is released
@@ -637,29 +833,18 @@ kgspBootstrapRiscvOSEarly_GH100
     else
     {
         NV_ASSERT_FAILED("Failed to boot GSP");
-        status = NV_ERR_NOT_READY;
-        goto exit;
+        return NV_ERR_NOT_READY;
     }
 
     NV_PRINTF(LEVEL_INFO, "Waiting for GSP fw RM to be ready...\n");
 
     // Link the status queue.
-    NV_ASSERT_OK_OR_GOTO(status, GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo),
-                          exit);
+    NV_ASSERT_OK_OR_RETURN(GspStatusQueueInit(pGpu, &pKernelGsp->pRpc->pMessageQueueInfo));
 
-    NV_ASSERT_OK_OR_GOTO(status, kgspWaitForRmInitDone(pGpu, pKernelGsp),
-                          exit);
+    NV_ASSERT_OK_OR_RETURN(kgspWaitForRmInitDone(pGpu, pKernelGsp));
 
     NV_PRINTF(LEVEL_INFO, "GSP FW RM ready.\n");
 
-exit:
-    // If GSP fails to boot, check if there's any DED error.
-    if (status != NV_OK)
-    {
-        kmemsysCheckEccCounts_HAL(pGpu, pKernelMemorySystem);
-    }
-    NV_ASSERT(status == NV_OK);
-
     return status;
 }
 
@@ -693,3 +878,133 @@ kgspGetGspRmBootUcodeStorage_GH100
         }
     kgspGetGspRmBootUcodeStorage_GA102(pGpu, pKernelGsp, ppBinStorageImage, ppBinStorageDesc);
 }
+
+NV_STATUS
+kgspIssueNotifyOp_GH100
+(
+    OBJGPU *pGpu,
+    KernelGsp *pKernelGsp,
+    NvU32 opCode,
+    NvU32 *pArgs,
+    NvU32 argc
+)
+{
+    NV_STATUS status = NV_OK;
+    NotifyOpSharedSurface *pNotifyOpSharedSurface;
+    volatile NvU32 *pInUse;
+    volatile NvU32 *pSeqAddr;
+    volatile NvU32 *pStatusAddr;
+    NvU32 *pOpCodeAddr;
+    NvU32 *pArgsAddr;
+    NvU32 *pArgcAddr;
+
+    NvU32 seqValue;
+    NvU32 i;
+    RMTIMEOUT timeout;
+
+    // 1. Validate the arguments.
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, opCode < GSP_NOTIFY_OP_OPCODE_MAX, NV_ERR_INVALID_ARGUMENT);
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, argc <= GSP_NOTIFY_OP_MAX_ARGUMENT_COUNT, NV_ERR_INVALID_ARGUMENT);
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, (argc == 0 || pArgs != NULL), NV_ERR_INVALID_ARGUMENT);
+
+    // 2. Set up ptrs to the shared memory.
+    NV_ASSERT_OR_RETURN(pKernelGsp->pNotifyOpSurf != NULL, NV_ERR_INVALID_STATE);
+
+    pNotifyOpSharedSurface = pKernelGsp->pNotifyOpSurf;
+
+    pInUse      = (NvU32*) &(pNotifyOpSharedSurface->inUse);
+    pSeqAddr    = (NvU32*) &(pNotifyOpSharedSurface->seqNum);
+    pOpCodeAddr = (NvU32*) &(pNotifyOpSharedSurface->opCode);
+    pStatusAddr = (NvU32*) &(pNotifyOpSharedSurface->status);
+    pArgcAddr   = (NvU32*) &(pNotifyOpSharedSurface->argc);
+    pArgsAddr   = (NvU32*) pNotifyOpSharedSurface->args;
+
+    while (!portAtomicCompareAndSwapU32(pInUse, 1, 0))
+    {
+        // We're not going to sleep, but safe to sleep also means safe to spin..
+        NV_ASSERT_OR_RETURN(portSyncExSafeToSleep(), NV_ERR_INVALID_STATE);
+        osSpinLoop();
+    }
+
+    // 3. Read current sequence counter value.
+    seqValue = *pSeqAddr;
+
+    // 4. Populate the opcode and arguments.
+    *pOpCodeAddr = opCode;
+    *pArgcAddr = argc;
+    for (i = 0; i < argc; i++)
+    {
+        pArgsAddr[i] = pArgs[i];
+    }
+
+    //
+    // Issue a store fence to ensure that the parameters have made it to memory
+    // before the interrupt is triggered.
+    //
+    portAtomicMemoryFenceStore();
+
+    // 5. Trigger notification interrupt to GSP.
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
+    GPU_VREG_WR32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_DOORBELL, NV_DOORBELL_NOTIFY_LEAF_SERVICE_LOCKLESS_OP_HANDLE);
+
+    // 6. Poll on the sequence number to ensure the op completed.
+    while (seqValue + 1 != *pSeqAddr)
+    {
+        status = gpuCheckTimeout(pGpu, &timeout);
+
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "gpuCheckTimeout failed, status = 0x%x\n", status);
+            goto error_ret;
+        }
+        osSpinLoop();
+    }
+
+    status = *pStatusAddr;
+
+error_ret:
+    portAtomicSetU32(pInUse, 0);
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, status);
+    return status;
+}
+
+NV_STATUS
+kgspCheckGspRmCcCleanup_GH100
+(
+    OBJGPU    *pGpu,
+    KernelGsp *pKernelGsp
+)
+{
+    KernelFalcon *pKernelFalcon   = staticCast(pKernelGsp, KernelFalcon);
+    NvU32         ccCleanupStatus = 0;
+    NV_STATUS     status          = NV_OK;
+
+    // Wait for GSP to explicitly state we have completed CC cleanup.
+    status = gpuTimeoutCondWait(pGpu, _kgspHasCcCleanupFinished, (void *)pKernelFalcon, NULL);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "CC secret cleanup failed due to timeout!\n");
+        goto exit;
+    }
+
+    // Now, check whether or not CC cleanup was successful.
+    ccCleanupStatus = kflcnRegRead_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1);
+    if (ccCleanupStatus == NV_SPDM_SECRET_TEARDOWN_SUCCESS)
+    {
+        // FIPS: Make status clear to user.
+        NV_PRINTF(LEVEL_ERROR, "CC secret cleanup successful!\n");
+    }
+    else
+    {
+        NV_PRINTF(LEVEL_ERROR, "CC secret cleanup failed! Status 0x%x\n",
+                  ccCleanupStatus);
+    }
+
+exit:
+
+    // Regardless of success or failure, write ack to GSP.
+    kflcnRegWrite_HAL(pGpu, pKernelFalcon, NV_PFALCON_FALCON_MAILBOX1, NV_SPDM_SECRET_TEARDOWN_ACK);
+
+    return status;
+}
