@@ -23,14 +23,11 @@
 
 #define NVOC_KERNEL_NVLINK_H_PRIVATE_ACCESS_ALLOWED
 
-// FIXME XXX
-#define NVOC_KERNEL_IOCTRL_H_PRIVATE_ACCESS_ALLOWED
-
 #include "os/os.h"
 #include "core/hal.h"
-#include "core/info_block.h"
 #include "core/locks.h"
 #include "core/thread_state.h"
+#include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gpu.h"
 
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
@@ -1494,6 +1491,16 @@ knvlinkCoreTrainingCompleteCallback
     }
 }
 
+NvlStatus
+knvlinkCoreGetCciLinkModeCallback
+(
+    nvlink_link *link,
+    NvU64       *mode
+)
+{
+    return NVL_SUCCESS;
+}
+
 /*
  * @brief nvlinkCoreGetUphyLoadCallback send ALI training on the specified link
  *
